pub enum EffectRange {
All,
Some(HashSet<Operand>),
}Variants
Implementations
Sourceimpl EffectRange
impl EffectRange
Sourcepub fn can_conflict(&self, another: &EffectRange, indvar: &Operand) -> bool
pub fn can_conflict(&self, another: &EffectRange, indvar: &Operand) -> bool
Check if two effect ranges conflict when parallelized. TODO-TLE: use a more efficient way to check conflict, separate in parallel_analysis
Sourcepub fn can_alias(&self, another: &EffectRange) -> bool
pub fn can_alias(&self, another: &EffectRange) -> bool
Check if two effect ranges can alias.
Sourcepub fn merge(&mut self, another: &EffectRange)
pub fn merge(&mut self, another: &EffectRange)
Merge two effect ranges.
Sourcepub fn get_single(&self) -> Option<&Operand>
pub fn get_single(&self) -> Option<&Operand>
Get the only operand if the effect range contains only one operand.