pub enum Node {
Entry(usize),
Normal(usize, Option<NodePtr>, Option<NodePtr>, InstPtr),
Phi(usize, Vec<(BBPtr, NodePtr)>, EffectRange),
}Expand description
Memory SSA node. Function in Node does not maintain use-def chain.
Variants
Entry(usize)
Entry(id) represents the memory state at the beginning of the function.
Normal(usize, Option<NodePtr>, Option<NodePtr>, InstPtr)
Normal(id, use_node, def_node, inst) represents a memory state after an instruction.
Phi(usize, Vec<(BBPtr, NodePtr)>, EffectRange)
Phi(id, args, range) represents a phi node.
Implementations
Sourceimpl Node
impl Node
Sourcepub fn get_use_node(&self) -> NodePtr
pub fn get_use_node(&self) -> NodePtr
Get use node. Use node is the node that is read from.
Sourcepub fn get_used_node(&self) -> Vec<NodePtr> ⓘ
pub fn get_used_node(&self) -> Vec<NodePtr> ⓘ
Get used nodes. Used nodes contains both use and def nodes.
Sourcepub fn get_used_node_mut(&mut self) -> Vec<&mut NodePtr> ⓘ
pub fn get_used_node_mut(&mut self) -> Vec<&mut NodePtr> ⓘ
Get mutable used nodes. Used nodes contains both use and def nodes.