pub struct MemorySSA<'a> {
pub effect_analysis: &'a EffectAnalysis,
/* private fields */
}Expand description
MemorySSA analyzer. Reference: https://llvm.org/docs/MemorySSA.html My version is different by analyzing the effect of function calls.
Fields
effect_analysis: &'a EffectAnalysisImplementations
Sourceimpl<'a> MemorySSA<'a>
impl<'a> MemorySSA<'a>
Sourcepub fn new(program: &Program, effect_analysis: &'a EffectAnalysis) -> Self
pub fn new(program: &Program, effect_analysis: &'a EffectAnalysis) -> Self
Build MemorySSA for program.
Sourcepub fn get_inst_node(&self, inst: InstPtr) -> Option<NodePtr>
pub fn get_inst_node(&self, inst: InstPtr) -> Option<NodePtr>
Get node from instruction.
Sourcepub fn get_block_node(&self, bb: BBPtr) -> Option<NodePtr>
pub fn get_block_node(&self, bb: BBPtr) -> Option<NodePtr>
Get node from block.
Sourcepub fn get_node_block(&self, node: NodePtr) -> Option<BBPtr>
pub fn get_node_block(&self, node: NodePtr) -> Option<BBPtr>
Get block from node.
Sourcepub fn predict_read(
&self,
src: NodePtr,
dst: InstPtr,
func: FunPtr,
) -> Result<Option<Operand>>
pub fn predict_read( &self, src: NodePtr, dst: InstPtr, func: FunPtr, ) -> Result<Option<Operand>>
Assume src writes memory, predict content read from dst.
Sourcepub fn remove_node(&mut self, node: NodePtr)
pub fn remove_node(&mut self, node: NodePtr)
Remove a node, update use-def chain.
Panics
Do not remove used phi node or entry node with this function!