Function

Struct Function 

pub struct Function {
    pub mem_pool: ObjPtr<IRBuilder>,
    pub name: String,
    pub entry: Option<BBPtr>,
    pub exit: Option<BBPtr>,
    pub return_type: ValueType,
    pub params: Vec<ParaPtr>,
}

Fields

mem_pool: ObjPtr<IRBuilder>name: Stringentry: Option<BBPtr>

Entry of function, if it is a function that is not defined in this module, it will be None. Such as library function.

exit: Option<BBPtr>

Exit of function, if it is a function that is not defined in this module, it will be None. Such as library function.

return_type: ValueTypeparams: Vec<ParaPtr>

BasicBlock of function parameters

Implementations

Source

impl Function

pub fn is_lib(&self) -> bool

Return true if it is a function that is not defined in this module.

pub fn is_main(&self) -> bool

Return true if it is main function.

pub fn dfs_iter(&self) -> DFSIterator

Create a depth-first iterator to traverse the graph structure of basicblocks. Traverse in the direction of data flow with the function entry as the starting point. Do not change the graph structure during traversal, which may cause unknown errors

pub fn bfs_iter(&self) -> BFSIterator

Create a breadth-first iterator to traverse the graph structure of basicblocks. Traverse in the direction of data flow with the function entry as the starting point. Do not change the graph structure during traversal, which may cause unknown errors

pub fn dfs_iter_rev(&self) -> DFSIteratorRev

Create a depth-first iterator to traverse the graph structure of basicblocks. Traverse in the reverse direction of data flow with the function exit as the starting point. Do not change the graph structure during traversal, which may cause unknown errors

pub fn bfs_iter_rev(&self) -> BFSIteratorRev

Create a breadth-first iterator to traverse the graph structure of basicblocks. Traverse in the reverse direction of data flow with the function exit as the starting point. Do not change the graph structure during traversal, which may cause unknown errors

pub fn po_iter(&self) -> POIterator

Create a postorder iterator to traverse the graph structure of basicblocks.

pub fn rpo_iter(&self) -> RPOIterator

Create a reverse postorder iterator to traverse the graph structure of basicblocks.

pub fn gen_llvm_ir(&self) -> String