ValueType

Enum ValueType 

pub enum ValueType {
    Void,
    SignedChar,
    Int,
    Float,
    Bool,
    Array(Box<ValueType>, usize),
    Pointer(Box<ValueType>),
}
Represent the type of a value.

Variants



Void



SignedChar



Int



Float



Bool



Array(Box<ValueType>, usize)



Pointer(Box<ValueType>)

Implementations

Source

impl ValueType

pub fn is_basic_type(&self) -> bool

pub fn is_num(&self) -> bool

pub fn is_pointer(&self) -> bool

pub fn is_array(&self) -> bool

pub fn size(&self) -> usize

Get size of this value type.

pub fn get_sub_type(&self) -> Option<&ValueType>

Get subtype of the value type. Subtype is the type of the element in the array or the type of the pointer.

pub fn get_base_type(&self) -> ValueType

Get base type of the value type. Base type is i32 / f32 for array.

pub fn default_initializer(&self) -> Result<Constant>

Get default initializer of this type.

pub fn to_precision_level(&self) -> i32

Convert a numeric value type to its precision level. Higher is more precise.

pub fn from_precision_level(level: i32) -> Self

Convert a precision level to a value type.

pub fn max_with(&self, b: &Self) -> Self

Max this type with another type, return more precise one. If types are not number, return void.