macro_rules! gen_common_code {
($type:ty,$id:ident) => { ... };
}Expand description
Use this macro to generate some code for instruction to simplify the codesize. Make sure to use this macro in the impl Instruction block and the Instruction struct must have a field named manager which is an instance of InstManager.