pub enum ValueType {
Void,
SignedChar,
Int,
Float,
Bool,
Array(Box<ValueType>, usize),
Pointer(Box<ValueType>),
}Expand description
Represent the type of a value.
Variants
Implementations
Sourceimpl ValueType
impl ValueType
pub fn is_basic_type(&self) -> bool
pub fn is_num(&self) -> bool
pub fn is_pointer(&self) -> bool
pub fn is_array(&self) -> bool
Sourcepub fn get_sub_type(&self) -> Option<&ValueType>
pub fn get_sub_type(&self) -> Option<&ValueType>
Get subtype of the value type. Subtype is the type of the element in the array or the type of the pointer.
Sourcepub fn get_base_type(&self) -> ValueType
pub fn get_base_type(&self) -> ValueType
Get base type of the value type. Base type is i32 / f32 for array.
Sourcepub fn default_initializer(&self) -> Result<Constant>
pub fn default_initializer(&self) -> Result<Constant>
Get default initializer of this type.
Sourcepub fn to_precision_level(&self) -> i32
pub fn to_precision_level(&self) -> i32
Convert a numeric value type to its precision level. Higher is more precise.
Sourcepub fn from_precision_level(level: i32) -> Self
pub fn from_precision_level(level: i32) -> Self
Convert a precision level to a value type.